; ================
; FROSTWAVE SPRITE
; ================

; Ram uses:

; $02   
; $00C2
; $1504
; $151C y positive flag
; $1528 x positive flag 
; $163E state timer
; $157C	x speed
; $1594	y speed
; $1602 animation and direction select
; $160E state

; =========
; CONSTANTS
; =========

!FROSTWAVE_SUMMON	= #$00
!FROSTWAVE_MOVING	= #$01
!FROSTWAVE_STOPPED	= #$02
!FROSTWAVE_SHATTER	= #$03
!FROSTWAVE_CARRIED	= #$04

!FROSTWAVE_SUMMONT	= #$40

!FROSTWAVE_TTSHATTER	= #$FF
!FROSTWAVE_STOPSPR	= #$03
!FROSTWAVE_SPEEDPOS	= #$40
!FROSTWAVE_SPEEDNEG	= #$90

; ===============
; INITIALIZIATION
; ===============

		print "INIT ",pc
		LDA !FROSTWAVE_SUMMON	; Initially moving state
		STA $160E,x		; Store move state
		LDA !FROSTWAVE_SUMMONT	; Store time until summon
		STA $163E,x		; is finished
		RTL

; ====================
; SPRITE MAIN FUNCTION
; ====================

		print "MAIN ",pc
		PHB			; Push Old Databank
		PHK			; Push Current Databank
		PLB			; Pull Current Databank
		JSR FROSTWAVE_GFX	; Display the sprite

		JSR SUB_OFF_SCREEN_X0	; Process Sprite of screen

		LDA $14C8,x		; Load Sprite State
		CMP #$08 		; Check for normal carried, stationable states
		BEQ PROCESS 		; Process sprite
		CMP #$09
		BEQ PROCESS
		CMP #$0B
		BEQ PROCESS
		BRA RETURN
PROCESS:	LDA $9D 		; Load sprite locking 
		BNE RETURN 		; Don't process sprite

		LDA $160E,x		; Load State
		CMP !FROSTWAVE_MOVING
		BEQ FROSTWAVE_MOVE
		CMP !FROSTWAVE_STOPPED
		BEQ FROSTWAVE_STOP
		CMP !FROSTWAVE_SHATTER
		BEQ FROSTWAVE_SHAT
		CMP !FROSTWAVE_SUMMON
		BEQ FROSTWAVE_SUMM
		CMP !FROSTWAVE_CARRIED
		BEQ FROSTWAVE_CRRY
		BRA RETURN

FROSTWAVE_MOVE:	JSR MoveFunc
		BRA RETURN
FROSTWAVE_STOP:	JSR StopFunc
		BRA RETURN
FROSTWAVE_SHAT:	JSR ShatterFunc
		BRA RETURN
FROSTWAVE_CRRY: JSR CarryFunc
		BRA RETURN
FROSTWAVE_SUMM:	LDA $163E,x
		CMP #$00
		BNE RETURN

		JSR ChangeFunc

RETURN:		PLB			; Pull Old Databank
		RTL

; =======================
; Change to Move Function
; =======================

; $1528 x positive flag 
; $151C y positive flag
; $157C	x speed
; $1594	y speed

ChangeFunc:	LDA !FROSTWAVE_MOVING
		STA $160E,x

		JSR SUB_HORZ_POS	; Get Horizontal Direction
		TYA
		STA $1528,x
		CMP #$00		; If 0 Mario is to the right
		BEQ MarioRight
MarioLeft:	LDA $E4,x
		SBC $94
		BRA CalcYSpeed
MarioRight:	LDA $94
		SBC $E4,x
CalcYSpeed:	STA $157C,x
		JSR SUB_VERT_POS	; Get Vertical Direction
		TYA
		STA $151C,x
		CMP #$01		; If 0 Mario is to the bottom
		BEQ MarioBottom

MarioTop:	LDA $14D4,x		; Load y sprite position (highbyte)
		XBA
		LDA $D8,x		; Load y sprite position (lowbyte)
		REP #$20		; Activate 16bit calculation
		SBC $96			; Substract Mario position
		SEP #$20		; Activate 8bit calculation
		BRA CalcFinalSpeed

MarioBottom:	LDA $14D4,x		; Load y sprite position (highbyte)
		XBA
		LDA $D8,x		; Load y sprite position (lowbyte)
		REP #$20		; Activate 16bit calculation
		SBC $96			; Substract Mario position
		EOR #$FFFF
		INC A
		SEP #$20		; Activate 8bit calculation

CalcFinalSpeed:	STA $1594,x
		ORA $157C,x
		AND #$80
		BPL SaveSpeeds
		LDA $157C,x
		LSR
		STA $157C,x
		LDA $1594,x
		LSR
		STA $1594,x
SaveSpeeds:	LDA $1528,x
		CMP #$00
		BEQ CheckYSpeed
		LDA $157C,x
		EOR #$FF
		INC A
		STA $157C,x
CheckYSpeed:	LDA $151C,x
		CMP #$00
		BEQ ChangeEnd
		LDA $1594,x
		EOR #$FF
		INC A
		STA $1594,x
ChangeEnd:	LDA #$0F		; Play sound effect
		STA $1DF9
		RTS

; =============
; Move Function
; =============

MoveFunc:	LDA $157C,x		; Load x speed
		STA $B6,x		; Store X Speed
		LDA $1594,x		; Load y speed
		STA $AA,x		; Store y speed
		JSL $01802A		; Move sprite
		JSL $01A7DC		; Check Mario interaction
		BCC NoMarioContact
		JSL $00F5B7		; Hurt Mario
ChangeShatter:	LDA !FROSTWAVE_SHATTER	; Change to shatter state
		STA $160E,x		
		RTS
NoMarioContact:	LDA $1588,x		; Get sprite direction
		AND #$0F		; Touching a wall?
		BNE ChangeShatter	; Yes, so shatter

		LDY #$0B                ; Cycle all other sprites
		PHX
SpriteLoop:	TYX
		LDA $7FAB9E,x		; Load Spritenumber
		CMP !FROSTWAVE_STOPSPR	; Check for frostblock
		BNE NextSprite		; Only process frostblocks
		LDA $160E,x
		CMP #$05
		BMI NextSprite

		JSL $03B6E5             ; Get Other Sprite Clipping
		PLX                     ; Pull X from Stack
		JSL $03B69F             ; Get Current Sprite Clipping
		JSL $03B72B             ; Check Clippings
		BCC NextSpritePush

	        LDA $1686,x		; Allow sprite interaction
                AND #$F7
	        STA $1686,x
		STZ $B6,x
		LDA #$10
		STA $AA,x
		STA $1594,x
		LDA !FROSTWAVE_STOPPED	; Load stopped state
		STA $160E,x		; Store stopped state
		LDA !FROSTWAVE_TTSHATTER
		STA $163E,x
		LDA #$0E 		; Play sound effect
		STA $1DF9

		RTS

NextSpritePush:	PHX
NextSprite:	DEY
		BPL SpriteLoop
		PLX
		RTS

; ================
; Stopped Function
; ================

StopFunc:	LDA $163E,x		; Check timer
		BNE HandleStop
		LDA !FROSTWAVE_SHATTER
		STA $160E,x
HandleStop:	JSL $01A7DC		; Check for Mario interaction
		BCC SkipMario
CheckCarry:	BIT $15
		BVC SkipMario
		LDA #$0B		; Set carried state
		STA $14C8,x
		LDA !FROSTWAVE_CARRIED
		STA $160E,x
		BRA SkipSolid
SkipMario:	JSL $01B44F		; Make Block Solid
SkipSolid:	JSL $01802A		; Move sprite
		RTS

; ==============
; Carry Function
; ==============

CarryFunc:	LDA $1588,x		; Get sprite direction
		AND #$0F		; Touching a wall?
		BEQ EndCarry
		LDA !FROSTWAVE_SHATTER	; Change to shatter state
		STA $160E,x
EndCarry:	RTS

; ================
; Shatter Function
; ================

ShatterFunc:	STZ $14C8,x		; Kill this sprite
		LDA $E4,x		; Copy X Position
		STA $9A
		LDA $14E0,x             ; High byte
		STA $9B
		LDA $D8,x               ; Copy Y Position
		STA $98
		LDA $14D4,x		; High byte
		STA $99

		PHB
		LDA #$02		; Setup Shatter arguments
		PHA
		PLB
		LDA #$00
		JSL $028663		; Call shatter routine
		PLB

		RTS

; ============
; GFX FUNCTION
; ============

FROSTWAVE_TILEMAP:		db $E0,$E6,$E4,$E2

FROSTWAVE_GFX:	JSR GET_DRAW_INFO	; Get necessary OAM Pointer, Y = OAM Index, X = Sprite Index
		LDA $00			; Get X Position
		STA $0300,y		; Store X Offset
		LDA $01			; Get Y Position
		STA $0301,y		; Store Y Offset

		LDA $15F6,x		; Get the palette from table
		ORA $64			; Take care of priorities
		STA $0303,y		; Store properity byte

		LDA $160E,x		; Get state
		CMP !FROSTWAVE_SUMMON	; Animate only if summoning
		BNE SkipAnimate

		LDA $163E,x
		LSR
		LSR
		LSR
		LSR
		BRA LoadTile
SkipAnimate:	LDA #$00
LoadTile:	PHX			; Push X to Stack
		TAX			; Transfer A to X
		LDA FROSTWAVE_TILEMAP,x	; Load current tile from Map
		PLX			; Pull X from Stack
		STA $0302,y		; Store current Frame tile
		INY			; Next Tile
		INY
		INY
		INY
		LDA #$00		; Load the amount of tiles
		LDY #$02		; Select 16x16 tiles drawing
		JSL $01B7B3		; Call tile render function
		RTS



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:             db $0C,$1C
SPR_T2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:		db $40,$B0
SPR_T13:		db $01,$FF
SPR_T14:		db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
			db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:		db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
			db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:  LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:  LDA #$0E                ;  |
STORE_03:	    STA $03		    ;  |            
		    BRA START_SUB	    ;  |
SUB_OFF_SCREEN_X0:  STZ $03		    ; /

START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return

VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:  LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS:		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
					LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
					SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
					SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
					STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
					LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
					SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
					BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
					INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16:				RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_VERT_POS
; This routine determines if Mario is above or below the sprite.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B829
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_VERT_POS:		LDY #$00               ;A:25A1 X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizCHC:0130 VC:085 00 FL:924
					LDA $96                ;A:25A1 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdiZCHC:0146 VC:085 00 FL:924
					SEC                    ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0170 VC:085 00 FL:924
					SBC $D8,x              ;A:2546 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizCHC:0184 VC:085 00 FL:924
					STA $0F                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0214 VC:085 00 FL:924
					LDA $97                ;A:25D6 X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0238 VC:085 00 FL:924
					SBC $14D4,x            ;A:2501 X:0007 Y:0000 D:0000 DB:03 S:01EA P:envMXdizcHC:0262 VC:085 00 FL:924
					BPL SPR_L11            ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0294 VC:085 00 FL:924
					INY                    ;A:25FF X:0007 Y:0000 D:0000 DB:03 S:01EA P:eNvMXdizcHC:0310 VC:085 00 FL:924
SPR_L11:				RTS                    ;A:25FF X:0007 Y:0001 D:0000 DB:03 S:01EA P:envMXdizcHC:0324 VC:085 00 FL:924

