; ===============
; PUSHWAVE SPRITE
; ===============

; Ram uses:

; $02   
; $00C2
; $1504 left speed
; $1528 right speed
; $1540 state timer
; $157C speedup timer
; $1594 mario direction
; $1602 animation select
; $160E state
; $163E 

; =========
; CONSTANTS
; =========

!PUSHWAVE_GONE		= #$00
!PUSHWAVE_FLATR		= #$01
!PUSHWAVE_FLATL		= #$02
!PUSHWAVE_FALL		= #$03
!PUSHWAVE_RISE		= #$04
!PUSHWAVE_BIG		= #$05
!PUSHWAVE_LEFT		= #$06
!PUSHWAVE_RIGHT		= #$07

!PUSHWAVE_GONETIME	= #$FF
!PUSHWAVE_FLATTIME	= #$10
!PUSHWAVE_RISETIME	= #$10
!PUSHWAVE_FALLTIME	= #$10
!PUSHWAVE_BIGTIME	= #$10

!PUSHWAVE_FLATANI	= #$00
!PUSHWAVE_MIDANI	= #$0A
!PUSHWAVE_HIGHANI	= #$14
!PUSHWAVE_RIGHTANI	= #$1E
!PUSHWAVE_LEFTANI	= #$28

!PUSHWAVE_LEFTSPEED	= #$F8
!PUSHWAVE_RGHTSPEED	= #$08
!PUSHWAVE_SPEEDUPD	= #$01
!PUSHWAVE_SPEEDUPT	= #$FF

!PUSHWAVE_FALLSPR	= #$01

; ===============
; INITIALIZIATION
; ===============

		print "INIT ",pc
		LDA !PUSHWAVE_GONE	; Initially flat state
		STA $160E,x		; Store flat state
		LDA !PUSHWAVE_GONETIME	; Store time until summon
		STA $1540,x		; is finished
		LDA !PUSHWAVE_FLATANI	; Load flat animation
		STA $1602,x		; Store flat animation
		LDA !PUSHWAVE_LEFTSPEED	; Load left speed
		STA $1504,x
		LDA !PUSHWAVE_RGHTSPEED
		STA $1528,x
		LDA !PUSHWAVE_SPEEDUPT
		STA $157C,x
		RTL

; ====================
; SPRITE MAIN FUNCTION
; ====================

		print "MAIN ",pc
		PHB			; Push Old Databank
		PHK			; Push Current Databank
		PLB			; Pull Current Databank

		LDA $160E,x
		CMP !PUSHWAVE_GONE
		BEQ PUSHWAVE_GONEL

		JSR HandleSpeedUp

		JSR PUSHWAVE_GFX	; Display the sprite

		LDA $14C8,x		; Load Sprite State
		CMP #$08 		; Check for normal State
		BNE RETURN		; Don't process sprite

PROCESS:	LDA $9D 		; Load sprite locking 
		BNE RETURN 		; Don't process sprite
		JSR SUB_OFF_SCREEN_X0	; Process Sprite of screen

		LDA $160E,x		; Load State
		CMP !PUSHWAVE_FLATR
		BEQ PUSHWAVE_FLTRL
		CMP !PUSHWAVE_FLATL
		BEQ PUSHWAVE_FLTLL
		CMP !PUSHWAVE_RISE
		BEQ PUSHWAVE_RISEL
		CMP !PUSHWAVE_FALL
		BEQ PUSHWAVE_FALLL

		JSR SUB_HORZ_POS	; Get direction
		TYA
		STA $1594,x		; and store it

		JSL $01A7DC		; Check Mario interaction
		BCC NoMarioContact

		LDA $1594,x
		CMP #$00
		BEQ PushRight
		LDA $1504,x
		ASL
		ORA #$80
		BRA CheckWater
PushRight:	LDA $1528,x
		ASL
CheckWater:	STA $7B
		LDA $75			; Load in water flag
		CMP #$00
		BEQ NoMarioContact
		LDA #$70		; We are touching Mario in Water
		STA $7D			; Pull him down


NoMarioContact:	JSR CheckSprites

		LDA $160E,x
		CMP !PUSHWAVE_BIG
		BEQ PUSHWAVE_BIGL
		CMP !PUSHWAVE_LEFT
		BEQ PUSHWAVE_LEFTL
		CMP !PUSHWAVE_RIGHT
		BEQ PUSHWAVE_RIGHL

RETURN:		PLB			; Pull Old Databank
		RTL


PUSHWAVE_GONEL:	JSR GoneFunc
		BRA RETURN
PUSHWAVE_FLTRL:	JSR FlatRiseFunc
		BRA RETURN
PUSHWAVE_FLTLL:	JSR FlatLowerFunc
		BRA RETURN
PUSHWAVE_RISEL:	JSR RiseFunc
		BRA RETURN
PUSHWAVE_FALLL:	JSR FallFunc
		BRA RETURN
PUSHWAVE_LEFTL:	JSR LeftFunc
		BRA RETURN
PUSHWAVE_RIGHL:	JSR RightFunc
		BRA RETURN
PUSHWAVE_BIGL:	JSR BigFunc
		BRA RETURN

; =============
; Gone Function
; =============

GoneFunc:	LDA $1540,x		; Load State timer
		CMP #$00
		BNE GoneEnd
		LDA !PUSHWAVE_FLATR
		STA $160E,x
		LDA !PUSHWAVE_FLATTIME
		STA $1540,x
GoneEnd:	RTS

; ==================
; Flat Rise Function
; ==================

FlatRiseFunc:	LDA $1540,x		; Load State timer
		CMP #$00
		BNE FlatRiseEnd
		LDA !PUSHWAVE_RISE
		STA $160E,x
		LDA !PUSHWAVE_RISETIME
		STA $1540,x
		LDA !PUSHWAVE_MIDANI
		STA $1602,x
FlatRiseEnd:	RTS

; ===================
; Flat Lower Function
; ===================

FlatLowerFunc:	LDA $1540,x		; Load State timer
		CMP #$00
		BNE FlatLowerEnd
		LDA !PUSHWAVE_GONE
		STA $160E,x
		LDA !PUSHWAVE_GONETIME
		STA $1540,x
FlatLowerEnd:	RTS

; =============
; Rise Function
; =============

RiseFunc:	LDA $1540,x		; Load State timer
		CMP #$00
		BNE RiseEnd
		LDA !PUSHWAVE_BIG
		STA $160E,x
		LDA !PUSHWAVE_BIGTIME
		STA $1540,x
		LDA !PUSHWAVE_HIGHANI
		STA $1602,x
RiseEnd:	RTS

; =============
; Fall Function
; =============

FallFunc:	LDA $1540,x		; Load State timer
		CMP #$00
		BNE FallEnd
		LDA !PUSHWAVE_FLATL
		STA $160E,x
		LDA !PUSHWAVE_FLATTIME
		STA $1540,x
		LDA !PUSHWAVE_FLATANI
		STA $1602,x
FallEnd:	RTS

; =============
; Left Function
; =============

LeftFunc:	LDA $1504,x		; Get Speed
		STA $B6,x
		STZ $AA,x
		JSL $01802A		; Update Position
		LDA $1594,x
		CMP #$01
		BEQ LeftEnd
		LDA !PUSHWAVE_BIG
		STA $160E,x
		LDA !PUSHWAVE_BIGTIME
		STA $1540,x
		LDA !PUSHWAVE_HIGHANI
		STA $1602,x
LeftEnd:	RTS

; ==============
; Right Function
; ==============

RightFunc:	LDA $1528,x		; Get Speed
		STA $B6,x
		STZ $AA,x
		JSL $01802A		; Update Position
		LDA $1594,x
		CMP #$00
		BEQ RightEnd
		LDA !PUSHWAVE_BIG
		STA $160E,x
		LDA !PUSHWAVE_BIGTIME
		STA $1540,x
		LDA !PUSHWAVE_HIGHANI
		STA $1602,x
RightEnd:	RTS

; ============
; Big Function
; ============

BigFunc:	LDA $1540,x		; Load State Timer
		CMP #$00
		BNE BigEnd
		LDA $1594,x
		CMP #$00
		BEQ BigRight
		LDA !PUSHWAVE_LEFT
		STA $160E,x
		LDA !PUSHWAVE_LEFTANI
		STA $1602,x
		RTS
BigRight:	LDA !PUSHWAVE_RIGHT
		STA $160E,x
		LDA !PUSHWAVE_RIGHTANI
		STA $1602,x
BigEnd:		RTS

; ===============
; Sprite FUNCTION
; ===============

CheckSprites:	LDY #$0B                ; Cycle all other sprites
		PHX                     ; Push x to Stack
SpriteLoop:	TYX                     ; Transfer Y to X
		LDA $7FAB9E,x		; Load Spritenumber
		CMP !PUSHWAVE_FALLSPR	; Check for frostblock
		BNE NextSprite
		LDA $14C8,x		; Load sprite state
		CMP #$08		; Only process active sprites
		BNE NextSprite
		
		JSL $03B6E5             ; Get Other Sprite Clipping
		PLX                     ; Pull X from Stack
		JSL $03B69F             ; Get Current Sprite Clipping
		JSL $03B72B             ; Check Clippings
		BCC NextSpritePush

		LDA !PUSHWAVE_FALL
		STA $160E,x
		LDA !PUSHWAVE_FALLTIME
		STA $1540,x
		LDA !PUSHWAVE_MIDANI
		STA $1602,x

		RTS

NextSpritePush:	PHX
NextSprite:	DEY
		BPL SpriteLoop
		PLX
		RTS

; ================
; Speedup FUNCTION
; ================

HandleSpeedUp:	LDA $157C,x		; Load speedup timer
		CMP #$00		; Compare to zero
		BNE SpeedUpRet
		LDA $1504,x		; Load left speed
		SEC
		SBC !PUSHWAVE_SPEEDUPD	; Increase speed
		STA $1504,x
		LDA $1528,x		; Load right speed
		CLC
		ADC !PUSHWAVE_SPEEDUPD	; Increase speed
		STA $1528,x
		LDA !PUSHWAVE_SPEEDUPT	; Reset speedup timer
SpeedUpRet:	DEC A			; Decrease speedup timer
		STA $157C,x
		RTS


; ============
; GFX FUNCTION
; ============

!PUSHWAVE_TILESCOUNT	=  #$09
PUSHWAVE_TILEMAP:
PUSHWAVE_FLAT:		db $86,$86,$86,$86,$86,$86,$80,$A0,$A0,$80
PUSHWAVE_MIDDLE:	db $86,$86,$86,$A2,$A2,$86,$C0,$C2,$C2,$C0
PUSHWAVE_HIGH:		db $82,$82,$A4,$84,$84,$A4,$C4,$C6,$C6,$C4
PUSHWAVE_RIGHT:		db $8A,$8C,$8E,$AA,$AC,$AE,$C8,$CA,$CC,$CE
PUSHWAVE_LEFT:		db $8E,$8C,$8A,$AE,$AC,$AA,$CE,$CC,$CA,$C8

PUSHWAVE_FLIPMAP:
PUSHWAVE_FLATFLIP:	db $00,$00,$00,$00,$00,$00,$00,$00,$40,$40
PUSHWAVE_MIDFLIP:	db $00,$00,$00,$00,$40,$00,$00,$00,$40,$40
PUSHWAVE_HIGHFLIP:	db $00,$40,$00,$00,$40,$40,$00,$00,$40,$40
PUSHWAVE_RIGHTFLIP:	db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00
PUSHWAVE_LEFTFLIP:	db $40,$40,$40,$40,$40,$40,$40,$40,$40,$40

PUSHWAVE_TILEXPOS:	
PUSHWAVE_FLATXPOS:	db $F8,$08,$E8,$F8,$08,$18,$E8,$F8,$08,$18
PUSHWAVE_MIDXPOS:	db $F8,$08,$E8,$F8,$08,$18,$E8,$F8,$08,$18
PUSHWAVE_HIGHXPOS:	db $F8,$08,$E8,$F8,$08,$18,$E8,$F8,$08,$18
PUSHWAVE_RIGHTXPOS:	db $F8,$08,$18,$F8,$08,$18,$E8,$F8,$08,$18
PUSHWAVE_LEFTXPOS:	db $E8,$F8,$08,$E8,$F8,$08,$E8,$F8,$08,$18

PUSHWAVE_TILEYPOS:
PUSHWAVE_FLATYPOS:	db $04,$04,$14,$14,$14,$14,$24,$24,$24,$24
PUSHWAVE_MIDYPOS:	db $04,$04,$14,$14,$14,$14,$24,$24,$24,$24
PUSHWAVE_HIGHYPOS:	db $04,$04,$14,$14,$14,$14,$24,$24,$24,$24
PUSHWAVE_RIGHTYPOS:	db $04,$04,$04,$14,$14,$14,$24,$24,$24,$24
PUSHWAVE_LEFTYPOS:	db $04,$04,$04,$14,$14,$14,$24,$24,$24,$24

PUSHWAVE_GFX:	JSR GET_DRAW_INFO		; Get necessary OAM Pointer, Y = OAM Index, X = Sprite Index
		LDA $1602,x			; Load animation Select Byte
		STA $02				; Store for processing
		PHX				; Push X to Stack, sprite index
		LDX !PUSHWAVE_TILESCOUNT	; X is loop counter, X = Loop Counter
TILE_LOOP:	PHX 				; Push X to Stack, loop index
		LDX $15E9			; Get current Sprite index
		LDA $15F6,x			; Get the palette from table
		ORA $64				; Take care of priorities
		
		PLX				; Pull X from Stack -> x loop index
		PHX				; loop index to Stack
		PHA				; half processed property byte to stack
		TXA				; get loop index for calculation
		CLC
		ADC $02				; add animation select byte
		TAX				; transfer to offset pointer
		PLA				; pull half processed property byte

		ORA PUSHWAVE_FLIPMAP,x		; Add in x flip flags
		STA $0303,y			; Store properity byte
		LDA PUSHWAVE_TILEMAP,x		; Load current tile from Map
		STA $0302,y			; Store current Frame tile

		LDA $00				; Get X Position
		CLC				; Clear Carry
		ADC PUSHWAVE_TILEXPOS,x		; Calculate X Offset
		STA $0300,y			; Store X Offset
		LDA $01				; Get Y Position
		CLC				; Clear Carry
		ADC PUSHWAVE_TILEYPOS,x		; Calculate Y Offset
		STA $0301,y			; Store Y Offset

		PLX				; Get loop index back

		INY				; Next Tile
		INY
		INY
		INY
		DEX				; Next Tile
		BPL TILE_LOOP			; Branch if X positive
		PLX				; Rull X from Stack
		LDA !PUSHWAVE_TILESCOUNT	; Load the amount of tiles
		LDY #$02			; Select 16x16 tiles drawing
		JSL $01B7B3			; Call tile render function
		RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:             db $0C,$1C
SPR_T2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC.w #$0040            ;  |
                    CMP.w #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC SPR_T1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA SPR_T2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
SPR_T12:		db $40,$B0
SPR_T13:		db $01,$FF
SPR_T14:		db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
			db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
SPR_T15:		db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
			db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SUB_OFF_SCREEN_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SUB_OFF_SCREEN_X2:  LDA #$04                ;  | 
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X7:  LDA #$0E                ;  |
STORE_03:	    STA $03		    ;  |            
		    BRA START_SUB	    ;  |
SUB_OFF_SCREEN_X0:  STZ $03		    ; /

START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ; | 
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; | 
                    CMP #$02                ; | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /    
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return

VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; | 
                    BCS RETURN_35           ; /
                    LDA $E4,x               ; \ 
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490

SUB_IS_OFF_SCREEN:  LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ; |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_HORZ_POS
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SUB_HORZ_POS:		LDY #$00				;A:25D0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1020 VC:097 00 FL:31642
					LDA $94					;A:25D0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZCHC:1036 VC:097 00 FL:31642
					SEC                     ;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1060 VC:097 00 FL:31642
					SBC $E4,x				;A:25F0 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizCHC:1074 VC:097 00 FL:31642
					STA $0F					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1104 VC:097 00 FL:31642
					LDA $95					;A:25F4 X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1128 VC:097 00 FL:31642
					SBC $14E0,x				;A:2500 X:0006 Y:0000 D:0000 DB:03 S:01ED P:envMXdiZcHC:1152 VC:097 00 FL:31642
					BPL SPR_L16             ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1184 VC:097 00 FL:31642
					INY                     ;A:25FF X:0006 Y:0000 D:0000 DB:03 S:01ED P:eNvMXdizcHC:1200 VC:097 00 FL:31642
SPR_L16:				RTS                     ;A:25FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:1214 VC:097 00 FL:31642
